#include "editor.h"

#include <qtextedit.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qstatusbar.h>
#include <qstylesheet.h>
#include <qapplication.h>

/* 
 *  Constructs an Editor which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'. 
 */
Editor::Editor( QWidget* parent,  const char* name, WFlags fl )
    : ljeditor( parent, name, fl )
{
    editField = ljeditor::TextEdit;
    fileName = "";
}

/*  
 *  Destroys the object and frees any allocated resources.
 */
Editor::~Editor()
{
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * Opens a new editor window.
 */
void Editor::fileNew()
{
    Editor * editor = new Editor;
    editor->show();
}

/* 
 * Opens a file in this editor window.
 */
void Editor::fileOpen()
{
    if ( ! saveAndContinue( tr( "Opening file aborted" ) ) )
	return;

    editField->setText( "" );
    setCaption( tr( "ljedit" ) );
    
	
    fileName = QFileDialog::getOpenFileName( ".", QString::null, this, 
                                             "opendialog", tr( "Open File ..." ) );

    if ( fileName.isEmpty() ){
	statusBar()->message( tr( "Opening file aborted" ), 2000 );
	return;
    }

    QFile file( fileName );
    if ( ! file.open( IO_ReadOnly ) ){
	statusBar()->message( QString( tr( "Can't read %1" ) ).arg( fileName ), 2000 );
	return;
    }

    QTextStream t( &file );
    QString text = t.read();
    if ( QStyleSheet::mightBeRichText( text ) )
	editField->setTextFormat( RichText );
    else
	editField->setTextFormat( PlainText );

    editField->setText( text );
    setCaption( fileName );
    statusBar()->message( QString( tr( "%1 loaded" ) ).arg( fileName ), 2000 );
}

/* 
 * Save the content of the editor in fileName.
 */
void Editor::fileSave()
{
    if ( fileName.isEmpty() ){
	fileSaveAs();
	return;
    } 

    QFile file( fileName );
    if ( ! file.open( IO_WriteOnly ) ) {
        statusBar()->message( QString( tr( "Could not write to %1" ) ).arg(fileName),
                                      2000 );
        return;
    }
    QTextStream t( &file );
    t << editField->text();
    file.close();

    setCaption( fileName );

    statusBar()->message( QString( tr( "File %1 saved" ) ).arg( fileName ), 2000 );
}

/* 
 * Get the filename to save the file.
 * Uses fileSave() for actually saving the file.
 * Stores information whether HTML or plain text should be written.  
 */
void Editor::fileSaveAs()
{
    QFileDialog * fileDialog = new QFileDialog( ".", QString::null, this, 
                                                "saveasdialog", TRUE );
    fileDialog->setMode( QFileDialog::AnyFile );

    QString htmlFilter = tr( "HTML files (*.html *.htm)" );                                            
    fileDialog->addFilter( tr( "Text files (*.txt)" ) ); 
    fileDialog->addFilter( htmlFilter );
    fileDialog->setCaption( tr( "Save As" ) );
    fileDialog->exec();

    fileName = fileDialog->selectedFile();

    if ( fileDialog->selectedFilter() == htmlFilter )
	editField->setTextFormat( RichText );
    else
	editField->setTextFormat( PlainText );

    if ( ! fileName.isEmpty() )
	fileSave();
    else 
        statusBar()->message( tr( "Saving aborted" ), 2000 );
}

/* 
 * What happens when a window close event occurs? 
 */
void Editor::closeEvent( QCloseEvent * close )
{
    if ( saveAndContinue( tr( "Closing aborted" ) ) )
	close->accept();
    else
        close->ignore();
}

/*
 * Ask user to save the old file or to cancel the operation. 
 */
bool Editor::saveAndContinue( QString abortMessage )
{
    if ( ! fileName.isEmpty() || ! editField->text().isEmpty() ){
	switch( QMessageBox::information( this, fileName,
                                         QString( tr( "Save %1?" ) ).arg( fileName ),
                                         tr( "Yes" ), tr( "No" ), tr( "Cancel" ) ) ){
        case 0: if ( ! fileName.isEmpty() )
                    fileSave();
                else
                    fileSaveAs();
                return TRUE; 
                break;
        case 1: statusBar()->message( QString( tr( "%1 not saved" ) ).arg(fileName),
                                      2000 );
                return TRUE;
                break;
        case 2: statusBar()->message( abortMessage, 2000 );
                return FALSE;
                break;
        default: return FALSE;
        }                    
    } else {
	return TRUE;
    }
}
